function MotionShiftIllusion_withAdjustment
%
%2015/09/22 Hisakata made

AssertOpenGL;

try
    inputfilename = input('File Name?','s');
    fname = sprintf('%s.mat',inputfilename);
    
    KbName('UnifyKeyNames');
    
    %=======Make a stracture==========
    Condition = [1 0 -1]; %Direction of motion
    Name_of_Condition = {'LeftMotion' 'Static' 'RightMotion'};
    Number_of_Condition = length(Condition);
    Number_of_Repeart = 10;
    Min_stimulus = -2; %position deg/ negative = left
    Max_stimulus = 2; %position deg/ positiove = right
    StepSize = 0.1; %position deg
    
    [Data,Randam_Order_Condition,Randam_Order_trial,Total_Number_of_Trial] =...
        MakeDataFile_Adjustment(Number_of_Repeart,Number_of_Condition,Name_of_Condition);
    
    Data_raw = zeros(Total_Number_of_Trial,4)-999;
    %==================================
    
    %========Open window and get the window size=======
    screenNumbers=Screen('Screens');
    [wp,Screen_rect] = Screen('OpenWindow',max(screenNumbers));
    FRAME = Screen('FrameRate',wp);
    xcenter = Screen_rect(3)/2;
    ycenter = Screen_rect(4)/2;
    screenwidth = Screen_rect(3);
    %==================================================
    
    if FRAME == 0
        FRAME = 60;
    end
    
    %==========Calculate Pixels/One degree==================
    ScreenWidth = 40;%cm
    Observation = (ScreenWidth/screenwidth)*57.3*60;
    Onedegcm = 1/(360/pi*(atan(1/Observation/2)));
    Onedegpix = screenwidth/(ScreenWidth/Onedegcm);
    %=======================================================
    
    %==========Set Black, White and Gray index============
    white = WhiteIndex(wp);
    black = BlackIndex(wp);
    gray = (white + black)/2;
    inc = white - gray;
    %=====================================================
    
    %=========Variables===================
    Eccentricity = 6;
    Position_of_Stimulus = [Eccentricity*Onedegpix 0]; % [x y]
    fixation_size = 10; %pix
    %=====================================
    
    %========Prepare the image============
    Gabor_sigma = 30/60 * Onedegpix;
    arraysize = 5*Onedegpix;%Gabor array size
    SF = 1 ;%cpd
    Velocity = 4;% deg/s
    Contrast = 1;
    TF = SF * Velocity; %Hz
    
    angle = pi/2; %Radian
    
    numFrames = round(FRAME*(1/TF));
    [x,y] = meshgrid(-arraysize/2:arraysize/2,-arraysize/2:arraysize/2);
    
    for MotionDirection = 1:Number_of_Condition
        for i = 1:numFrames
            phase = Condition(MotionDirection)*(i/numFrames)*2*pi;
            f = (SF/Onedegpix)*2*pi;
            a = cos(angle)*f;b = sin(angle)*f;
            m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
            m = gray + inc*m';
            
            Stimulus_texture(MotionDirection,i) = Screen('MakeTexture',wp,m);
        end
    end
    
    %Draw Reference Gabor
    phase = 0;
    f = (SF/Onedegpix)*2*pi;
    a = cos(angle)*f;b = sin(angle)*f;
    m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
    m = gray + inc*m';
    Ref_texture = Screen('MakeTexture',wp,m);
    
    HightPixels_texture = arraysize;
    WidthPixels_texture = arraysize;
    %====================================
    
    %=======Calculate the presentation position===========
    stimulus_xcenter = xcenter + Position_of_Stimulus(1);
    stimulus_ycenter = ycenter + Position_of_Stimulus(2);
    stimulus_center_rect = [stimulus_xcenter stimulus_ycenter stimulus_xcenter stimulus_ycenter];
    stimulus_rect = [0 0 WidthPixels_texture HightPixels_texture];
    reference_rect = [0 0 WidthPixels_texture HightPixels_texture];
    
    upper_Reference_xcenter = xcenter + Position_of_Stimulus(1);
    upper_Reference_ycenter = ycenter - HightPixels_texture;
    upper_Reference_center_rect = [upper_Reference_xcenter upper_Reference_ycenter...
        upper_Reference_xcenter upper_Reference_ycenter];
    
    lower_Reference_xcenter = xcenter + Position_of_Stimulus(1);
    lower_Reference_ycenter = ycenter + HightPixels_texture;
    lower_Reference_center_rect = [lower_Reference_xcenter lower_Reference_ycenter...
        lower_Reference_xcenter lower_Reference_ycenter];
    
    upper_reference_position_rect = upper_Reference_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    lower_reference_position_rect = lower_Reference_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    
    fixation_rect = [xcenter-fixation_size/2 ycenter-fixation_size/2 xcenter+fixation_size/2 ycenter+fixation_size/2];
    %=====================================================
    
    %=======Change degree to pixels ======================
    Min_stimulus = Min_stimulus*Onedegpix;
    Max_stimulus =  Max_stimulus*Onedegpix;
    StepSize = StepSize*Onedegpix;
    %=====================================================
    
%========= Begin the Experiment ===========================================
    HideCursor;
    
    for ii = 1:Total_Number_of_Trial
        
        Current_Condition = Randam_Order_Condition(ii);
        Current_trial = Randam_Order_trial(ii);
        
        Stimulus = rand(1)*(abs(Max_stimulus)+abs(Min_stimulus))-abs(Min_stimulus);
        
        stimulus_position_rect = stimulus_center_rect+ ...
                [-WidthPixels_texture/2+Stimulus -HightPixels_texture/2 WidthPixels_texture/2+Stimulus HightPixels_texture/2];
        
        %~~~~ Presentation and Get the answer~~~~~~~~~~~~~~~~~~
        k = 1;
        Onset_time = GetSecs;
        Screen('FillRect',wp,[127 127 127]);
        Screen('FillArc',wp,black,fixation_rect,0,360);
        while 1
            Screen('FillArc',wp,black,fixation_rect,0,360);
            Screen('DrawTexture',wp,Ref_texture,reference_rect,upper_reference_position_rect);
            Screen('DrawTexture',wp,Ref_texture,reference_rect,lower_reference_position_rect);
            Screen('DrawTexture',wp,Stimulus_texture(Current_Condition,k),stimulus_rect,stimulus_position_rect);
            Screen('Flip',wp);
            
            [keyIsDown, secs, keyCode] = KbCheck;
            if keyIsDown
                if strcmp(KbName(keyCode),'f')
                    Stimulus = Stimulus - StepSize;
                    if Stimulus < Min_stimulus
                        Stimulus = Min_stimulus;
                    end
                elseif strcmp(KbName(keyCode),'j')
                    Stimulus = Stimulus + StepSize;
                    if Stimulus > Max_stimulus
                        Stimulus = Max_stimulus;
                    end
                elseif strcmp(KbName(keyCode),'space')
                    
                    break
                end
            end
            
            stimulus_position_rect = stimulus_center_rect+ ...
                [-WidthPixels_texture/2+Stimulus -HightPixels_texture/2 ...
                WidthPixels_texture/2+Stimulus HightPixels_texture/2];
            
            k = k + 1;
            if k > numFrames
                k = 1;
            end 
        end
        %Calculate the reaction time
        Offset_time = GetSecs;
        ReactionTime = Offset_time - Onset_time;
        %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        
        Screen('FillRect',wp,[127 127 127]);
        Screen('FillArc',wp,black,fixation_rect,0,360);
        Screen('Flip',wp);
        
        %~1) Put the answer into the matrix
        Data_raw(ii,1) = ii;
        Data_raw(ii,2) = Current_Condition;
        Data_raw(ii,3) = Stimulus;
        Data_raw(ii,4) = ReactionTime;
        
        %~2) Put the answer into the stracture
        Data(Current_Condition).AnswerSheet(Current_trial) = Stimulus;
        Data(Current_Condition).ReactionTime(Current_trial) = ReactionTime;
        
        WaitSecs(0.5);
    end
%===============End the Experiment=========================================
    
    save(fname,'Data_raw','Data');
    ShowCursor;
    Screen('CloseAll');
    
catch ME
    ShowCursor;
    Screen('CloseAll');
    rethrow(ME);
end;

%% local function
function [Data,Randam_Order_Condition,Randam_Order_trial,Total_Number_of_Trial] =...
    MakeDataFile_Adjustment(Number_of_Repeart,Number_of_Condition,Name_of_Condition)

Answer_Sheet = zeros(1,Number_of_Repeart)-999;
Reaction_Time = zeros(1,Number_of_Repeart)-999;

for i = 1:Number_of_Condition
    Data(i).Condition = Name_of_Condition(i);
    Data(i).AnswerSheet = Answer_Sheet;
    Data(i).ReactionTime = Reaction_Time;
end

Total_Number_of_Trial = Number_of_Condition*Number_of_Repeart;

Randam_condition_Number = randperm(Total_Number_of_Trial);

Randam_Order_Condition = floor((Randam_condition_Number-1)/Number_of_Repeart)+1;
Randam_Order_trial = mod((Randam_condition_Number-1),Number_of_Repeart)+1;
