function MotionShiftIllusion_withConstant
%
%2015/09/22 Hisakata made

AssertOpenGL;

try    
    inputfilename = input('File Name?','s');
    fname = sprintf('%s.mat',inputfilename);
    
    %======= Make data set ========
    Condition = [1 0 -1]; %Direction of motion 
    Name_of_Condition = {'LeftMotion' 'Static' 'RightMotion'};
    Number_of_Condition = length(Condition);
    Stimulus = [-1 -0.5 0 0.5 1];
    Number_of_Repeart = 10;
    
    [Data,Total_Number_of_Trial,Condition_Number,Row_Number,Column_Number] =...
        ConstantStimuli(Condition,Stimulus,Number_of_Repeart,Name_of_Condition);
    Data_raw = zeros(Total_Number_of_Trial,5)-999;
    %==============================
    
    %========Open window and get the window size=======
    screenNumbers=Screen('Screens');
    [wp,Screen_rect] = Screen('OpenWindow',max(screenNumbers));
    FRAME = Screen('FrameRate',wp);
    xcenter = Screen_rect(3)/2;
    ycenter = Screen_rect(4)/2;
    screenwidth = Screen_rect(3);
    %==================================================
    
    if FRAME == 0
        FRAME = 60;
    end
    
    %==========Calculate Pixels/One degree==================
    ScreenWidth = 40;%cm
    Observation = (ScreenWidth/screenwidth)*57.3*60;
    Onedegcm = 1/(360/pi*(atan(1/Observation/2)));
    Onedegpix = screenwidth/(ScreenWidth/Onedegcm);
    %=======================================================
    
    %==========Set Black, White and Gray index============
    white = WhiteIndex(wp);
    black = BlackIndex(wp);
    gray = (white + black)/2;
    inc = white - gray;
    %=====================================================
    
    %=========Variables===================
    Eccentricity = 6;
    Position_of_Stimulus = [Eccentricity*Onedegpix 0]; % [x y]
    fixation_size = 10; %pix
    %=====================================
    
    %========Prepare the image============
    Gabor_sigma = 30/60 * Onedegpix;
    arraysize = 5*Onedegpix;%Gabor array size
    SF = 1 ;%cpd
    Velocity = 4;% deg/s
    Contrast = 1;
    TF = SF * Velocity; %Hz
    
    angle = pi/2; %Radian
    
    movieDurationSecs = 1;%seconds
    movieDurationFrames = movieDurationSecs * FRAME;
    numFrames = round(FRAME*(1/TF));
    movieFrameIndices = mod(0:(movieDurationFrames-1),numFrames)+1;
    
    [x,y] = meshgrid(-arraysize/2:arraysize/2,-arraysize/2:arraysize/2);
    
    for MotionDirection = 1:Number_of_Condition
        for i = 1:numFrames
            phase = Condition(MotionDirection)*(i/numFrames)*2*pi;
            f = (SF/Onedegpix)*2*pi;
            a = cos(angle)*f;b = sin(angle)*f;
            m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
            m = gray + inc*m';
            
            Stimulus_texture(MotionDirection,i) = Screen('MakeTexture',wp,m);
        end
    end
    
    %Draw Reference Gabor
    phase = 0;
    f = (SF/Onedegpix)*2*pi;
    a = cos(angle)*f;b = sin(angle)*f;
    m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
    m = gray + inc*m';
    Ref_texture = Screen('MakeTexture',wp,m);
    
    HightPixels_texture = arraysize;
    WidthPixels_texture = arraysize;
    %====================================
    
    %=======Calculate the presentation position===========
    stimulus_xcenter = xcenter + Position_of_Stimulus(1);
    stimulus_ycenter = ycenter + Position_of_Stimulus(2);
    stimulus_center_rect = [stimulus_xcenter stimulus_ycenter stimulus_xcenter stimulus_ycenter];
    stimulus_rect = [0 0 WidthPixels_texture HightPixels_texture];
    reference_rect = [0 0 WidthPixels_texture HightPixels_texture];
    
    upper_Reference_xcenter = xcenter + Position_of_Stimulus(1);
    upper_Reference_ycenter = ycenter - HightPixels_texture;
    upper_Reference_center_rect = [upper_Reference_xcenter upper_Reference_ycenter ...
        upper_Reference_xcenter upper_Reference_ycenter];
    
    lower_Reference_xcenter = xcenter + Position_of_Stimulus(1);
    lower_Reference_ycenter = ycenter + HightPixels_texture;
    lower_Reference_center_rect = [lower_Reference_xcenter lower_Reference_ycenter ...
        lower_Reference_xcenter lower_Reference_ycenter];
    
    upper_reference_position_rect = upper_Reference_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    lower_reference_position_rect = lower_Reference_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    
    fixation_rect = [xcenter-fixation_size/2 ycenter-fixation_size/2 xcenter+fixation_size/2 ycenter+fixation_size/2];
    %=====================================================
    
    %=======Change degree to pixels ======================
     Stimulus = Stimulus * Onedegpix;
    %=====================================================
    
%========= Begin the Experiment ===========================================
HideCursor;

for ii = 1:Total_Number_of_Trial

    Current_RowNumber = Row_Number(ii);
    Current_ColumnNumber = Column_Number(ii);
    Current_Condition = Condition_Number(ii);
    Current_Stimulus = Stimulus(Current_RowNumber);

    stimulus_position_rect = stimulus_center_rect+ ...
        [-WidthPixels_texture/2+Current_Stimulus -HightPixels_texture/2 ...
        WidthPixels_texture/2+Current_Stimulus HightPixels_texture/2];
    
    %~~~~ Presentation ~~~~~~~~~~~~~~~~~~~~~~~    
    Onset_time = GetSecs;
    for k = 1:movieDurationFrames
        Screen('FillRect',wp,[127 127 127]);
        Screen('FillArc',wp,black,fixation_rect,0,360);
        Screen('DrawTexture',wp,Ref_texture,reference_rect,upper_reference_position_rect);
        Screen('DrawTexture',wp,Ref_texture,reference_rect,lower_reference_position_rect);
        Screen('DrawTexture',wp,Stimulus_texture(Current_Condition,movieFrameIndices(k)),...
            stimulus_rect,stimulus_position_rect);
        Screen('Flip',wp);
    end
    %Calculate the presentation time
    Offset_time = GetSecs;
    RealPresentationTime(ii) = Offset_time - Onset_time;
    
    Screen('FillRect',wp,[127 127 127]);
    Screen('FillArc',wp,black,fixation_rect,0,360);
    Screen('Flip',wp);
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    %~~~~ Get the answer ~~~~~~~~~~~~~~~~~~~~~
    Onset_time = GetSecs;
    while 1
        [keyIsDown, secs, keyCode] = KbCheck;
        if keyIsDown
            if strcmp(KbName(keyCode),'f') %Answering Left
                Behavior = 0;
                break
            elseif strcmp(KbName(keyCode),'j') %Answering Right
                Behavior = 1;
                break
            end
        end
    end
    %Calculate the reaction time
    Offset_time = GetSecs;
    ReactionTime = Offset_time - Onset_time;
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    %~1) Put the answer into the matrix
    Data_raw(ii,1) = ii;
    Data_raw(ii,2) = Current_Condition;
    Data_raw(ii,3) = Current_Stimulus;
    Data_raw(ii,4) = Behavior;
    Data_raw(ii,5) = ReactionTime;

    %~2) Put the answer into the stracture
    Data(Current_Condition).AnswerSheet(Current_RowNumber,Current_ColumnNumber) = Behavior;
    Data(Current_Condition).PresentationOrder(Current_RowNumber,Current_ColumnNumber) = ii;
    
    WaitSecs(0.5);
end
%===============End the Experiment=========================================

save(fname,'Data_raw','Data');
ShowCursor;
Screen('CloseAll');
    
catch ME
    ShowCursor;
    Screen('CloseAll');
    rethrow(ME);
end;


%% local function
function [Data,Total_Number_of_Trial,Condition_Number,Row_Number,Column_Number] =...
    ConstantStimuli(Condition,Stimulus,Number_of_Repeart,Name_of_Condition)

Number_of_Condition = length(Condition);
Number_of_Stimulus = length(Stimulus);

%====Make the answer sheet =========
Answer_Sheet = zeros(Number_of_Stimulus,Number_of_Repeart+1)-999;
Answer_Sheet(:,1) = Stimulus;
Presentation = Answer_Sheet;
%===================================

%====Make Data set =================
for i = 1:Number_of_Condition
    Data(i).Condition = Name_of_Condition(i);
    Data(i).AnswerSheet = Answer_Sheet;
    Data(i).PresentationOrder = Presentation;
end
%===================================

Total_Number_of_Trial = Number_of_Condition * Number_of_Stimulus * Number_of_Repeart;
Randam_condition_Number = randperm(Total_Number_of_Trial)-1;

Condition_Number = floor((Randam_condition_Number)/(Number_of_Stimulus*Number_of_Repeart))+1;
TrialNumber_inCondition = mod((Randam_condition_Number),(Number_of_Stimulus*Number_of_Repeart));
Row_Number = floor(TrialNumber_inCondition/Number_of_Repeart)+1;
Column_Number = mod(TrialNumber_inCondition,Number_of_Repeart)+1+1;
