function MotionShiftIllusion_withUpDown
%
%2015/09/22 Hisakata made

AssertOpenGL;

try
    inputfilename = input('File Name?','s');
    fname = sprintf('%s.mat',inputfilename);
    
    %=======Make a stracture========
    Condition = [1 0 -1]; %Direction of motion                           
    Name_of_Condition = {'LeftMotion' 'Static' 'RightMotion'};
    Number_of_Condition = length(Condition);
    
    Min_stimulus = -2;    %position deg/ negative = left                          
    Max_stimulus = 2;     %position deg/ negative = right                         
    Start_stimulus = 0;   %position deg/ zero = vertical position                          
    StepSize = 0.1;      %position deg                    
    TotalRunNumber = 10;                             
    
    for i = 1:Number_of_Condition
        Data(i).Condition = Name_of_Condition(i);  
        Data(i).Answer = -999;                      
        Data(i).Stimulus = Start_stimulus;          
        Data(i).NextStimulus = Start_stimulus;      
        Data(i).RunNumber = 0;                      
        Data(i).TrialNumber = 0;                    
    end
    %================================
    
    %========Open window and get the window size=======
    screenNumbers=Screen('Screens');
    [wp,Screen_rect] = Screen('OpenWindow',max(screenNumbers));
    FRAME = Screen('FrameRate',wp);
    xcenter = Screen_rect(3)/2;
    ycenter = Screen_rect(4)/2;
    screenwidth = Screen_rect(3);
    %==================================================
    
    if FRAME == 0
        FRAME = 60;
    end
    
    %==========Calculate Pixels/One degree==================
    ScreenWidth = 40;%cm
    Observation = (ScreenWidth/screenwidth)*57.3*60;
    Onedegcm = 1/(360/pi*(atan(1/Observation/2)));
    Onedegpix = screenwidth/(ScreenWidth/Onedegcm);
    %=======================================================
    
    %==========Set Black, White and Gray index============
    white = WhiteIndex(wp);
    black = BlackIndex(wp);
    gray = (white + black)/2;
    inc = white - gray;
    %=====================================================
    
    %=========Variables===================
    Eccentricity = 6;
    Position_of_Stimulus = [Eccentricity*Onedegpix 0]; % [x y]
    fixation_size = 10; %pix
    %=====================================
    
    %========Prepare the image============
    Gabor_sigma = 30/60 * Onedegpix;
    arraysize = 5*Onedegpix;%Gabor array size
    SF = 1 ;%cpd
    Velocity = 4;% deg/s
    Contrast = 1;
    TF = SF * Velocity; %Hz
    
    angle = pi/2; %Radian
    
    movieDurationSecs = 1;%seconds
    movieDurationFrames = movieDurationSecs * FRAME;
    numFrames = round(FRAME*(1/TF));
    movieFrameIndices = mod(0:(movieDurationFrames-1),numFrames)+1;
    
    [x,y] = meshgrid(-arraysize/2:arraysize/2,-arraysize/2:arraysize/2);
    
    for MotionDirection = 1:Number_of_Condition
        for i = 1:numFrames
            phase = Condition(MotionDirection)*(i/numFrames)*2*pi;
            f = (SF/Onedegpix)*2*pi;
            a = cos(angle)*f;b = sin(angle)*f;
            m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
            m = gray + inc*m';
            
            Stimulus_texture(MotionDirection,i) = Screen('MakeTexture',wp,m);
        end
    end
    
    %Draw Reference Gabor
    phase = 0;
    f = (SF/Onedegpix)*2*pi;
    a = cos(angle)*f;b = sin(angle)*f;
    m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
    m = gray + inc*m';
    Ref_texture = Screen('MakeTexture',wp,m);
    
    HightPixels_texture = arraysize;
    WidthPixels_texture = arraysize;
    %====================================
    
    %=======Calculate the presentation position===========
    stimulus_xcenter = xcenter + Position_of_Stimulus(1);
    stimulus_ycenter = ycenter + Position_of_Stimulus(2);
    stimulus_center_rect = [stimulus_xcenter stimulus_ycenter stimulus_xcenter stimulus_ycenter];
    stimulus_rect = [0 0 WidthPixels_texture HightPixels_texture];
    reference_rect = [0 0 WidthPixels_texture HightPixels_texture];
    
    upper_Reference_xcenter = xcenter + Position_of_Stimulus(1);
    upper_Reference_ycenter = ycenter - HightPixels_texture;
    upper_Reference_center_rect = [upper_Reference_xcenter upper_Reference_ycenter ...
        upper_Reference_xcenter upper_Reference_ycenter];
    
    lower_Reference_xcenter = xcenter + Position_of_Stimulus(1);
    lower_Reference_ycenter = ycenter + HightPixels_texture;
    lower_Reference_center_rect = [lower_Reference_xcenter lower_Reference_ycenter ...
        lower_Reference_xcenter lower_Reference_ycenter];
    
    upper_reference_position_rect = upper_Reference_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    lower_reference_position_rect = lower_Reference_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    
    fixation_rect = [xcenter-fixation_size/2 ycenter-fixation_size/2 xcenter+fixation_size/2 ycenter+fixation_size/2];  
    %=====================================================
    
    
    %=======Change degree to pixels ======================
    Min_stimulus = Min_stimulus * Onedegpix;                        
    Max_stimulus = Max_stimulus * Onedegpix;                              
    Start_stimulus = Start_stimulus * Onedegpix;                             
    StepSize = StepSize * Onedegpix;
    
    for i = 1:Number_of_Condition
        Data(i).Condition = Name_of_Condition(i);
        Data(i).Answer = -999;
        Data(i).Stimulus = Start_stimulus;
        Data(i).NextStimulus = Start_stimulus;
        Data(i).RunNumber = 0;
        Data(i).TrialNumber = 0;
    end
    %=====================================================
    
%========= Begin the Experiment ===========================================
HideCursor;
while 1

    RandamTrialNumber_inCondition = 5;
    RandamTrials = RandamTrialNumber_inCondition * Number_of_Condition;
    TrialOrder = randperm(RandamTrials);
    ConditionOrder = floor((TrialOrder-1)/RandamTrialNumber_inCondition)+1;

    for ii = 1:RandamTrials
        Current_Condition = ConditionOrder(ii);

        Current_TrialNumber = Data(Current_Condition).TrialNumber+1;
        Current_Stimulus = Data(Current_Condition).NextStimulus;
        
        stimulus_position_rect = stimulus_center_rect+ ...
            [-WidthPixels_texture/2+Current_Stimulus -HightPixels_texture/2 ...
            WidthPixels_texture/2+Current_Stimulus HightPixels_texture/2];
        
        if Data(Current_Condition).RunNumber == TotalRunNumber
            ;
        else
            
            %~~~~ Presentation ~~~~~~~~~~~~~~~~~~
            Onset_time = GetSecs;
            for k = 1:movieDurationFrames
                Screen('FillRect',wp,[127 127 127]);
                Screen('FillArc',wp,black,fixation_rect,0,360);
                Screen('DrawTexture',wp,Ref_texture,reference_rect,upper_reference_position_rect);
                Screen('DrawTexture',wp,Ref_texture,reference_rect,lower_reference_position_rect);
                Screen('DrawTexture',wp,Stimulus_texture(Current_Condition,movieFrameIndices(k)),...
                    stimulus_rect,stimulus_position_rect);
                Screen('Flip',wp);
            end
            %Calculate the presentation time
            Offset_time = GetSecs;
            RealPresentationTime(ii) = Offset_time - Onset_time;
            
            Screen('FillRect',wp,[127 127 127]);
            Screen('FillArc',wp,black,fixation_rect,0,360);
            Screen('Flip',wp);
            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            
            %~~~~ Get the answer ~~~~~~~~~~~~~~~~~~~~~
            Onset_time = GetSecs;
            while 1
                [keyIsDown, secs, keyCode] = KbCheck;
                if keyIsDown
                    if strcmp(KbName(keyCode),'f') %Answering Left
                        Behavior = 0;
                        break
                    elseif strcmp(KbName(keyCode),'j') %Answering Right
                        Behavior = 1;
                        break
                    end
                end
            end
            %Calculate the reaction time
            Offset_time = GetSecs;
            ReactionTime = Offset_time - Onset_time;
            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            [Next_Stimulus,RunNumber] = OneUp_OneDown(Data,Current_Condition,...
                Current_TrialNumber,Current_Stimulus,Behavior,StepSize,Min_stimulus,Max_stimulus);

            Data(Current_Condition).Answer(Current_TrialNumber) = Behavior;
            Data(Current_Condition).Stimulus(Current_TrialNumber) = Current_Stimulus;
            Data(Current_Condition).NextStimulus = Next_Stimulus;
            Data(Current_Condition).RunNumber = RunNumber;
            Data(Current_Condition).TrialNumber = Current_TrialNumber;
        end
    end

    RunNumber_for_Ending = TotalRunNumber*Number_of_Condition;
    Corrent_TotalRunNumber = 0;
    for iii = 1:Number_of_Condition
        Corrent_TotalRunNumber = Corrent_TotalRunNumber + Data(iii).RunNumber;
    end
    if Corrent_TotalRunNumber == RunNumber_for_Ending
        break
    end
end
%===============End the Experiment=========================================
    
save(fname,'Data');
ShowCursor;
Screen('CloseAll');

catch ME
    ShowCursor;
    Screen('CloseAll');
    rethrow(ME);
end;

%% local function
function [Next_Stimulus,RunNumber] = OneUp_OneDown(Data,Current_Condition,...
    Current_TrialNumber,Current_Stimulus,Behavior,StepSize,Min_stimulus,Max_stimulus)

if Behavior == 0
    Next_Stimulus = Current_Stimulus+StepSize;
    if Next_Stimulus > Max_stimulus
        Next_Stimulus = Max_stimulus;
    end
elseif Behavior == 1
    Next_Stimulus = Current_Stimulus-StepSize;
    if Next_Stimulus < Min_stimulus
        Next_Stimulus = Min_stimulus;
    end
end

if Current_TrialNumber == 1
    RunNumber = 0;
else
    PreviousBehavior = Data(Current_Condition).Answer(Current_TrialNumber-1);
    if Behavior == PreviousBehavior
        RunNumber = Data(Current_Condition).RunNumber;
    else
        RunNumber = Data(Current_Condition).RunNumber + 1;
    end
end