function Rating_Experiment
%
%2015/07/06 Hisakata made
%2015/07/13 Hisakata update

AssertOpenGL;

try   
    fname = sprintf('Experiment.mat');
    
    KbName('UnifyKeyNames');
    
    %=======Make a structure========
    Condition = [1 2 3];
    Name_of_Condition = {'Cond1' 'Cond2' 'Cond3'};
    Number_of_Condition = length(Condition);
    Number_of_Repeart = 5;
    
    [Data,Randam_Order_Condition,Randam_Order_trial,Total_Number_of_Trial] =...
        MakeDataFile_Rating(Number_of_Repeart,Number_of_Condition,Name_of_Condition);
    
    Data_raw = zeros(Total_Number_of_Trial,4)-999;
    %==================================
    
%========= Begin the Experiment ===========================================

%========Open window and get the window size=======
screenNumbers=Screen('Screens');
[wp,Screen_rect] = Screen('OpenWindow',max(screenNumbers));
xcenter = Screen_rect(3)/2;
ycenter = Screen_rect(4)/2;
screenwidth = Screen_rect(3);
Screen('TextSize', wp, 20);
%==================================================

%==========Set Black, White and Gray index============
white = WhiteIndex(wp);
black = BlackIndex(wp);
%=====================================================

%========Define Scale Parameters===================
Max_rating = 7;
label1 = 'x';
label2 = '';
ScaleWidth = round(screenwidth/2);
ScaleHalfWidth = round(ScaleWidth/2);
ScaleHeight = 20;
ScaleHalfHeight = round(ScaleHeight/2);

ScaleCenterX = xcenter;
ScaleCenterY = ycenter + 100;

ScaleSpace = double(ScaleWidth)/(Max_rating-1);
background = white;
%=====================================================

HideCursor;

for ii = 1:Total_Number_of_Trial

    Current_Condition = Randam_Order_Condition(ii);
    Current_trial = Randam_Order_trial(ii);

    Onset_time = GetSecs;
    Current_Rating = 3;
    
    while 1
        %~~~~ Presentation ~~~~~~~~~~~~~~~~~~
        Screen('FillRect',wp,background);
        
        % Put Stimulus 
        Screen('DrawText', wp, Name_of_Condition{Current_Condition}, xcenter-50, ycenter-100, black ,background);
        
        % Show Scale
        Screen('DrawText', wp, double(label1), ScaleCenterX-ScaleHalfWidth-20, ScaleCenterY-80, black ,background);
        Screen('DrawText', wp, double(label2), ScaleCenterX+ScaleHalfWidth-20, ScaleCenterY-80, black ,background);
        Screen('DrawLine',wp, black, ScaleCenterX-ScaleHalfWidth, ScaleCenterY,ScaleCenterX+ScaleHalfWidth, ScaleCenterY, 3);
        for i=1:Max_rating
            tmp_x = ScaleCenterX-ScaleHalfWidth+round(ScaleSpace*(i-1));
            Screen('DrawLine',wp, black, tmp_x, ScaleCenterY-ScaleHalfHeight, tmp_x, ScaleCenterY+ScaleHalfHeight, 3);
            if i==Current_Rating
                pointlist = [tmp_x-5,ScaleCenterY-30
                             tmp_x,ScaleCenterY-20
                             tmp_x+5,ScaleCenterY-30];
                Screen('FillPoly',wp,black,pointlist);
            end
        end
        
        Screen('Flip',wp);
        %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        
        %~~~~ Get the answer ~~~~~~~~~~~~~~~~~~~~~
        [keyIsDown, secs, keyCode] = KbCheck;
        if keyIsDown
        
            if keyCode(KbName('space'))
                break
            elseif keyCode(KbName('RightArrow'))
                Current_Rating = Current_Rating+1;
                if Current_Rating > Max_rating
                    Current_Rating = Max_rating;
                end
            elseif keyCode(KbName('LeftArrow'))
                Current_Rating = Current_Rating-1;
                if Current_Rating < 1
                    Current_Rating = 1;
                end
            end
        
            WaitSecs(0.1);
        end

        %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    end

    %Calculate the reaction time
    Offset_time = GetSecs;
    ReactionTime = Offset_time - Onset_time;

    %~1) Put the answer into the matrix
    Data_raw(ii,1) = ii;
    Data_raw(ii,2) = Current_Condition;
    Data_raw(ii,3) = Current_Rating;
    Data_raw(ii,4) = ReactionTime;

    %~2) Put the answer into the structure
    Data(Current_Condition).AnswerSheet(Current_trial) = Current_Rating;
    Data(Current_Condition).ReactionTime(Current_trial) = ReactionTime;
    
    Screen('FillRect',wp,background);
    Screen('Flip',wp);
    WaitSecs(0.3);
end
%===============End the Experiment=========================================

save(fname,'Data_raw','Data');
ShowCursor;
Screen('CloseAll');

catch ME
    ShowCursor;
    Screen('CloseAll');
    rethrow(ME);
end;

%% local function
function [Data,Randam_Order_Condition,Randam_Order_trial,Total_Number_of_Trial] =...
    MakeDataFile_Rating(Number_of_Repeart,Number_of_Condition,Name_of_Condition)

Answer_Sheet = zeros(1,Number_of_Repeart)-999;
Reaction_Time = zeros(1,Number_of_Repeart)-999;

for i = 1:Number_of_Condition
    Data(i).Condition = Name_of_Condition(i);   
    Data(i).AnswerSheet = Answer_Sheet;         
    Data(i).ReactionTime = Reaction_Time;       
end

Total_Number_of_Trial = Number_of_Condition*Number_of_Repeart;

Randam_condition_Number = randperm(Total_Number_of_Trial);

Randam_Order_Condition = floor((Randam_condition_Number-1)/Number_of_Repeart)+1;
Randam_Order_trial = mod((Randam_condition_Number-1),Number_of_Repeart)+1;
