function SimpleGaborMotion_Demo
%
%2015/09/22 Hisakata made

AssertOpenGL;

try 
    input('Please press Enter');
    
    %========Open window and get the window size=======
    screenNumbers=Screen('Screens');
    [wp,Screen_rect] = Screen('OpenWindow',max(screenNumbers));
    FRAME = Screen('FrameRate',wp);
    xcenter = Screen_rect(3)/2;
    ycenter = Screen_rect(4)/2;
    screenwidth = Screen_rect(3);
    %==================================================
    
    if FRAME == 0
        FRAME = 60;
    end
    
    %=========Variables===================
    Number_of_Presentation = 1;
    Position_of_Stimulus = [0 0]; % [x y]
    %=====================================
    
    %==========Calculate Pixels/One degree==================
    ScreenWidth = 40;%cm
    Observation = (ScreenWidth/screenwidth)*57.3*60;
    Onedegcm = 1/(360/pi*(atan(1/Observation/2)));
    Onedegpix = screenwidth/(ScreenWidth/Onedegcm);
    %=======================================================
    
    %==========Set Black, White and Gray index============
    white = WhiteIndex(wp);
    black = BlackIndex(wp);
    gray = (white + black)/2;
    inc = white - gray;
    %=====================================================
    
    %========Prepare the image============
    Gabor_sigma = 60/60 * Onedegpix;
    arraysize = 7*Onedegpix;%Gabor array size
    SF = 1 ;%cpd
    Velocity = 1;% deg/s
    Contrast = 1;
    TF = SF * Velocity; %Hz
        
    direc = 1; %Directin of Gabor
    angle = pi/2; %Radian
    
    movieDurationSecs = 1;%seconds
    movieDurationFrames = movieDurationSecs * FRAME;
    numFrames = round(FRAME*(1/TF));
    movieFrameIndices = mod(0:(movieDurationFrames-1),numFrames)+1;

    
    [x,y] = meshgrid(-arraysize/2:arraysize/2,-arraysize/2:arraysize/2);
    for i = 1:numFrames
        phase = direc*(i/numFrames)*2*pi;
        f = (SF/Onedegpix)*2*pi;
        a = cos(angle)*f;b = sin(angle)*f;
        m = Contrast*cos(a*x+b*y+phase).* exp(-(x.^2/(2*Gabor_sigma.^2)+ y.^2/(2*Gabor_sigma.^2)));
        m = gray + inc*m';

        Stimulus_texture(i) = Screen('MakeTexture',wp,m);
    end
    
    HightPixels_texture = arraysize;
    WidthPixels_texture = arraysize;
    %====================================

    %=======Calculate the presentation position===========
    stimulus_xcenter = xcenter + Position_of_Stimulus(1);
    stimulus_ycenter = ycenter + Position_of_Stimulus(2);
    stimulus_center_rect = [stimulus_xcenter stimulus_ycenter stimulus_xcenter stimulus_ycenter];
    stimulus_rect = [0 0 WidthPixels_texture HightPixels_texture];
    
    stimulus_position_rect = stimulus_center_rect+ ...
        [-WidthPixels_texture/2 -HightPixels_texture/2 WidthPixels_texture/2 HightPixels_texture/2];
    %=====================================================

    RealPresentationTime = zeros(1,Number_of_Presentation) -999;
 
    %~~~~~~~~~~~~~~~~~Presentation~~~~~~~~~~~~~~~~~~~~~~~
    HideCursor;
    
    for ii = 1:Number_of_Presentation    
        Screen('FillRect',wp,[127 127 127]);  
        
        Onset_time = GetSecs;
        for k = 1:movieDurationFrames
            Screen('DrawTexture',wp,Stimulus_texture(movieFrameIndices(k)),stimulus_rect,stimulus_position_rect);
            Screen('Flip',wp);
%             Screen('WaitBlanking',wp);
        end
        %Calculate the presentation time
        Offset_time = GetSecs;
        RealPresentationTime(ii) = Offset_time - Onset_time;            
        
        Screen('FillRect',wp,[127 127 127]);        
        Screen('Flip',wp);
        
        WaitSecs(1); 
    end
    %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    %=====Close the window====================
    ShowCursor;
    Screen('CloseAll');
    %=========================================

    disp(RealPresentationTime);
    
catch ME
    ShowCursor;
    Screen('CloseAll');
    rethrow(ME);
end;
